@@ -3,17 +3,37 @@ import locale
 import os
 import requests
 from flask import Flask, current_app, g
-from flask.ext.sqlalchemy import SQLAlchemy
+from flask.ext import sqlalchemy
 from flask.ext.wtf.csrf import CsrfProtect
+from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.schema import MetaData
 
 
-db = SQLAlchemy()
+db = sqlalchemy.SQLAlchemy()
+# Patch Flask-SQLAlchemy to use a custom Metadata instance with a naming scheme
+# for constraints.
+def _patch_metadata():
+    naming_convention = {
+        'fk': ('fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s'
+                '_%(referred_column_0_name)s'),
+        'pk': 'pk_%(table_name)s',
+        'ix': 'ix_%(table_name)s_%(column_0_name)s',
+        'ck': 'ck_%(table_name)s_%(constraint_name)s',
+        'uq': 'uq_%(table_name)s_%(column_0_name)s',
+    }
+    metadata = MetaData(naming_convention=naming_convention)
+    base = declarative_base(cls=sqlalchemy.Model, name='Model',
+                            metaclass=sqlalchemy._BoundDeclarativeMeta,
+                            metadata=metadata)
+    base.query = sqlalchemy._QueryProperty(db)
+    db.Model = base
+_patch_metadata()
 
 
 from .util import DB_STATS, AcceptRequest
 
 
-__version__ = u'0.9.2'
+__version__ = u'0.9.3'
 
 
 requests_session = requests.Session()
