@@ -7,5 +7,5 @@ from .request import AcceptRequest
 from .sqlstats import DB_STATS
 from .unistr import unistr, ensure_unicode
 from .urlparse import urlparse, urlunparse
-from .utc import utc
+from .datetime import utc, DateTime
 from .xmlify import xmlify
